<?php

namespace Drupal\commerce_giftcard\Plugin\Commerce\CheckoutPane;

use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_checkout\Plugin\Commerce\CheckoutFlow\CheckoutFlowInterface;
use Drupal\commerce_checkout\Plugin\Commerce\CheckoutPane\CheckoutPaneBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the gift card redemption pane.
 *
 * @CommerceCheckoutPane(
 *   id = "commerce_giftcard_redemption",
 *   label = @Translation("Giftcard redemption"),
 *   default_step = "_sidebar",
 *   wrapper_element = "container",
 * )
 */
class GiftcardRedemption extends CheckoutPaneBase {

  /**
   * The inline form manager.
   *
   * @var \Drupal\commerce\InlineFormManager
   */
  protected $inlineFormManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition, ?CheckoutFlowInterface $checkout_flow = NULL) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition, $checkout_flow);
    $instance->inlineFormManager = $container->get('plugin.manager.commerce_inline_form');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'allow_multiple' => FALSE,
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationSummary() {
    if ($this->configuration['allow_multiple']) {
      $summary = $this->t('Allows multiple gift cards: Yes');
    }
    else {
      $summary = $this->t('Allows multiple gift cards: No');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['allow_multiple'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allow multiple gift cards to be redeemed'),
      '#default_value' => $this->configuration['allow_multiple'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['allow_multiple'] = $values['allow_multiple'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildPaneForm(array $pane_form, FormStateInterface $form_state, array &$complete_form) {
    $inline_form = $this->inlineFormManager->createInstance('commerce_giftcard_redemption', [
      'order_id' => $this->order->id(),
      'max_giftcards' => $this->configuration['allow_multiple'] ? NULL : 1,
    ]);

    $pane_form['form'] = [
      '#parents' => array_merge($pane_form['#parents'], ['form']),
    ];
    $pane_form['form'] = $inline_form->buildInlineForm($pane_form['form'], $form_state);

    return $pane_form;
  }

  /**
   * {@inheritdoc}
   */
  public function validatePaneForm(array &$pane_form, FormStateInterface $form_state, array &$complete_form) {
    /** @var \Drupal\commerce_order\Adjustment[] $adjustments */
    $adjustments = $this->order->getAdjustments(['commerce_giftcard']);
    $storage = $this->entityTypeManager->getStorage('commerce_giftcard');

    foreach ($adjustments as $adjustment) {
      $giftcard = $storage->load($adjustment->getSourceId());
      if (!$giftcard) {
        $form_state->setError($pane_form, $this->t('Gift card %code is not available anymore.', ['%code' => $giftcard->getCode()]));
      }
      $balance = $giftcard->getBalance();
      if (!$balance) {
        $form_state->setError($pane_form, $this->t('Unable to check gift card %code balance, please try again.', ['%code' => $giftcard->getCode()]));
      }
      if ($balance->lessThan($adjustment->getAmount()->multiply('-1'))) {
        $form_state->setError($pane_form, $this->t('Gift card %code balance has been modified, please check it again.', ['%code' => $giftcard->getCode()]));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitPaneForm(array &$pane_form, FormStateInterface $form_state, array &$complete_form) {
    // The form was submitted with a non-applied gift card in the input field,
    // mapped to a gift card ID in GiftcardRedemptionForm::validateForm().
    if (!empty($pane_form['form']['code']['#giftcard_id'])) {
      $this->order->get('commerce_giftcards')->appendItem($pane_form['form']['code']['#giftcard_id']);
    }
  }

}
